<?php
/**
 * @package     Com_Localise
 * @subpackage  Controller
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */
namespace Joomla\Component\Localise\Administrator\Controller;

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Router\Route;

/**
 * Controller class for the localise component
 *
 * @package     Extensions.Components
 * @subpackage  Localise
 * @since       1.0
 */
class DisplayController extends BaseController
{
	/**
	 * Method to display a view.
	 *
	 * @param   boolean  $cachable   If true, the view output will be cached
	 * @param   array    $urlparams  An array of safe url parameters and their variable types, for valid values see {@link JFilterInput::clean()}.
	 *
	 * @return  JController  This object to support chaining.
	 *
	 * @since   1.5
	 */
	public function display($cachable = false, $urlparams = false)
	{
		$vName  = $this->input->get('view', 'languages');
		$layout = $this->input->get('layout', 'default');
		$id     = $this->input->getInt('id');

		if ($vName == 'translations')
		{
			$view     = $this->getView('translations', 'html');
			$packages = $this->getModel('Packages', 'Administrator', array('ignore_request' => true));
			$view->setModel($packages);
		}
		// Check for edit form.
		elseif ($vName == 'language' && $layout == 'edit'
			&& !$this->checkEditId('com_localise.edit.language', $id))
		{
			// Somehow the person just went to the form - we don't allow that.
			$this->setMessage(Text::sprintf('JLIB_APPLICATION_ERROR_UNHELD_ID', $id));
			$this->setRedirect(Route::_('index.php?option=com_localise&view=languages', false));

			return false;
		}
		elseif ($vName == 'translation' && ($layout == 'edit' || $layout == 'raw')
			&& !$this->checkEditId('com_localise.edit.translation', $id))
		{
			// Somehow the person just went to the form - we don't allow that.
			$this->setMessage(Text::sprintf('JLIB_APPLICATION_ERROR_UNHELD_ID', $id));
			$this->setRedirect(Route::_('index.php?option=com_localise&view=translations', false));

			return false;
		}
		elseif ($vName == 'packagefile' && $layout == 'edit'
			&& !$this->checkEditId('com_localise.edit.packagefile', $id))
		{
			// Somehow the person just went to the form - we don't allow that.
			$this->setMessage(Text::sprintf('JLIB_APPLICATION_ERROR_UNHELD_ID', $id));
			$this->setRedirect(Route::_('index.php?option=com_localise&view=packages', false));

			return false;
		}
		elseif ($vName == 'package' && $layout == 'edit'
			&& !$this->checkEditId('com_localise.edit.package', $id))
		{
			// Somehow the person just went to the form - we don't allow that.
			$this->setMessage(Text::sprintf('JLIB_APPLICATION_ERROR_UNHELD_ID', $id));
			$this->setRedirect(Route::_('index.php?option=com_localise&view=packages', false));

			return false;
		}
		else
		{
			$this->input->set('view', $vName);
		}

		parent::display($cachable, $urlparams);
	}
}
